@@ -1,5 +1,5 @@
 /* nm.c -- Describe symbol table of a rel file.
-   Copyright (C) 1991-2015 Free Software Foundation, Inc.
+   Copyright (C) 1991-2016 Free Software Foundation, Inc.
 
    This file is part of GNU Binutils.
 
@@ -28,7 +28,15 @@
 #include "libiberty.h"
 #include "elf-bfd.h"
 #include "elf/common.h"
+#define DO_NOT_DEFINE_AOUTHDR
+#define DO_NOT_DEFINE_FILHDR
+#define DO_NOT_DEFINE_LINENO
+#define DO_NOT_DEFINE_SCNHDR
+#include "coff/external.h"
+#include "coff/internal.h"
+#include "libcoff.h"
 #include "bucomm.h"
+#include "plugin-api.h"
 #include "plugin.h"
 
 /* When sorting by size, we use this structure to hold the size and a
@@ -56,6 +64,7 @@ struct extended_symbol_info
   symbol_info *sinfo;
   bfd_vma ssize;
   elf_symbol_type *elfinfo;
+  coff_symbol_type *coffinfo;
   /* FIXME: We should add more fields for Type, Line, Section.  */
 };
 #define SYM_NAME(sym)        (sym->sinfo->name)
@@ -331,7 +340,7 @@ set_output_format (char *f)
 }
 
 static const char *
-get_symbol_type (unsigned int type)
+get_elf_symbol_type (unsigned int type)
 {
   static char buff [32];
 
@@ -354,6 +363,32 @@ get_symbol_type (unsigned int type)
       return buff;
     }
 }
+
+static const char *
+get_coff_symbol_type (const struct internal_syment *sym)
+{
+  static char buff [32];
+
+  switch (sym->n_sclass)
+    {
+    case C_BLOCK: return "Block";
+    case C_FILE:  return "File";
+    case C_LINE:  return "Line";
+    }
+
+  if (!sym->n_type)
+    return "None";
+    
+  switch (DTYPE(sym->n_type))
+    {
+    case DT_FCN: return "Function";
+    case DT_PTR: return "Pointer";
+    case DT_ARY: return "Array";
+    }
+  
+  sprintf (buff, _("<unknown>: %d/%d"), sym->n_sclass, sym->n_type);
+  return buff;
+}
 
 /* Print symbol name NAME, read from ABFD, with printf format FORM,
    demangling it if requested.  */
@@ -823,8 +858,17 @@ print_symbol (bfd *        abfd,
 
   info.sinfo = &syminfo;
   info.ssize = ssize;
-  /* Synthetic symbols do not have a full elf_symbol_type set of data available.  */
-  info.elfinfo = is_synthetic ? NULL : elf_symbol_from (abfd, sym);
+  /* Synthetic symbols do not have a full symbol type set of data available.  */
+  if (is_synthetic)
+    {
+      info.elfinfo = NULL;
+      info.coffinfo = NULL;
+    }
+  else
+    {
+      info.elfinfo = elf_symbol_from (abfd, sym);
+      info.coffinfo = coff_symbol_from (sym);
+    }
 
   format->print_symbol_info (&info, abfd);
 
@@ -985,7 +1029,7 @@ print_size_symbols (bfd *              abfd,
    SYMCOUNT is the number of symbols in MINISYMS and SYNTH_COUNT
    is the number of these that are synthetic.  Synthetic symbols,
    if any are present, always come at the end of the MINISYMS.
-   
+
    SIZE is the size of a symbol in MINISYMS.  */
 
 static void
@@ -1059,7 +1103,7 @@ display_rel_file (bfd *abfd, bfd *archive_bfd)
       non_fatal (_("%s: no symbols"), bfd_get_filename (abfd));
       return;
     }
-  
+
   if (show_synthetic && size == sizeof (asymbol *))
     {
       asymbol *synthsyms;
@@ -1514,7 +1558,10 @@ print_symbol_info_sysv (struct extended_symbol_info *info, bfd *abfd)
       /* Type, Size, Line, Section */
       if (info->elfinfo)
 	printf ("%18s|",
-		get_symbol_type (ELF_ST_TYPE (info->elfinfo->internal_elf_sym.st_info)));
+		get_elf_symbol_type (ELF_ST_TYPE (info->elfinfo->internal_elf_sym.st_info)));
+      else if (info->coffinfo)
+	printf ("%18s|",
+		get_coff_symbol_type (&info->coffinfo->native->u.syment));
       else
 	printf ("                  |");
 
@@ -1530,6 +1577,8 @@ print_symbol_info_sysv (struct extended_symbol_info *info, bfd *abfd)
 
       if (info->elfinfo)
 	printf("|     |%s", info->elfinfo->symbol.section->name);
+      else if (info->coffinfo)
+	printf("|     |%s", info->coffinfo->symbol.section->name);
       else
 	printf("|     |");
     }
@@ -1570,6 +1619,7 @@ main (int argc, char **argv)
 
   program_name = *argv;
   xmalloc_set_program_name (program_name);
+  bfd_set_error_program_name (program_name);
 #if BFD_SUPPORTS_PLUGINS
   bfd_plugin_set_program_name (program_name);
 #endif
