@@ -1,45 +1,23 @@
-/* Target dependent code for ARC processor family, for GDB, the GNU debugger.
+/* Target dependent code for GNU/Linux ARC.
 
-   Copyright 2005 Free Software Foundation, Inc.
-   Copyright 2009-2015 Synopsys Inc.
-
-   Contributor Anton Kolesov <Anton.Kolesov@synopsys.com>
-   Contributor Jeremy Bennett <jeremy.bennett@embecosm.com> on behalf of
-   Synopsys Inc.
-   Contributed by Codito Technologies Pvt. Ltd. (www.codito.com) on behalf of
-   Synopsys Inc.
-
-   Authors: 
-      Jeremy Bennett       <jeremy.bennett@embecosm.com>
-      Soam Vasani          <soam.vasani@codito.com>
-      Ramana Radhakrishnan <ramana.radhakrishnan@codito.com> 
-      Richard Stuckey      <richard.stuckey@arc.com>
-      Anton Kolesov        <Anton.Kolesov@synopsys.com>
+   Copyright 2005-2016 Free Software Foundation, Inc.
+   Contributed by Synopsys Inc.
 
    This file is part of GDB.
-   
+
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.
-  
+
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
-  
+
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
-/******************************************************************************/
-/*                                                                            */
-/* Outline:                                                                   */
-/*     This module provides support for the ARC processor family's target     */
-/*     dependencies which are specific to the arc-linux-uclibc configuration  */
-/*     of the ARC gdb.                                                        */
-/*                                                                            */
-/******************************************************************************/
-
 /* gdb header files */
 #include "defs.h"
 #include "gdbarch.h"
@@ -51,9 +29,7 @@
 #include "solib-svr4.h"
 #include "block.h"
 #include "regset.h"
-#include "dis-asm.h"
 #include "opcode/arc.h"
-#include "opcodes/arc-dis-old.h"
 #include "exceptions.h"
 /* lookup_minimal_symbol */
 #include "minsyms.h"
@@ -63,6 +39,8 @@
 #include "symtab.h"
 /* delete_single_step_breakpoints */
 #include "gdbthread.h"
+/* linux_init_abi */
+#include "linux-tdep.h"
 
 /* ARC header files */
 #include "arc-tdep.h"
@@ -283,102 +261,305 @@ static const int arc_linux_core_reg_offsets[] = {
 /*                               local functions                              */
 /* -------------------------------------------------------------------------- */
 
-/*! Returns TRUE if the instruction at PC is a branch (of any kind).
 
-    We always set a breakpoint at the next instruction, even if we have an
-    unconditional branch, which means it won't be executed.
+/*! Extracts the result of the last condition checking from status32 register by
+    condition code.
 
-    @todo This doesn't work if we are looking at a delay slot. We need to do
-          something about that.
-
-    @param[out] fall_thru  Set to the address of the next insn.
-    @param[out] target     Set to the branch target. */
+  @param[in] condition_code	It must be a condition code of the last
+				condition that was checked and its result was
+				stored in status32 register, otherwise this
+				function may return the wrong value.
+  @param[in] status32		Status32 register's value.
+  @return  The result of condition checking if given condition code exists,
+	   otherwise throws the assertion error.*/
 static int
-arc_linux_next_pc (CORE_ADDR pc, CORE_ADDR *fall_thru, CORE_ADDR *target)
+arc_linux_is_condition_true (ULONGEST status32, int condition_code)
 {
-  struct regcache *regcache = get_current_regcache ();
-  struct gdbarch *gdbarch = get_regcache_arch (regcache);
-  struct disassemble_info di;
-  struct arcDisState instr;
-  int two_targets = FALSE;
-  ULONGEST  lp_end, lp_start, lp_count, status32;
+  int z_flag, n_flag, c_flag, v_flag;
 
-  /* so what is the instruction at the given pc? */
-  arc_initialize_disassembler (gdbarch, &di);
-  instr = arcAnalyzeInstr (pc, &di);
+  gdb_assert (condition_code >= 0 && condition_code <= 15);
 
-  /* by default, the next instruction is the one immediately after the one at
-     pc */
-  *fall_thru = pc + instr.instructionLen;
+  z_flag = status32 & ARC_REG_STATUS32_Z;
+  n_flag = status32 & ARC_REG_STATUS32_N;
+  c_flag = status32 & ARC_REG_STATUS32_C;
+  v_flag = status32 & ARC_REG_STATUS32_V;
+
+  switch (condition_code)
+    {
+    /* Always (unconditional branch/jump instruction). */
+    case 0:
+      return TRUE;
+
+    /* Zero (equals). */
+    case 1:
+      return z_flag;
+
+    /* Non-Zero (not equals). */
+    case 2:
+      return !z_flag;
+
+    /* Positive. */
+    case 3:
+      return !n_flag;
+
+    /* Negative. */
+    case 4:
+      return n_flag;
+
+    /* carry set, lower than (unsigned). */
+    case 5:
+      return c_flag;
+
+    /* Carry clear, higher or same (unsigned). */
+    case 6:
+      return !c_flag;
+
+    /* Over-flow set. */
+    case 7:
+      return v_flag;
+
+    /* Over-flow clear. */
+    case 8:
+      return !v_flag;
+
+    /* Greater than (signed). */
+    case 9:
+      return (n_flag && v_flag && !z_flag) || (!n_flag && !v_flag && !z_flag);
+
+    /* Greater than or equal to (signed). */
+    case 10:
+      return (n_flag && v_flag) || (!n_flag && !v_flag);
+
+    /* Less than (signed). */
+    case 11:
+      return (n_flag && !v_flag) || (!n_flag && v_flag);
+
+    /* Less than or equal to (signed). */
+    case 12:
+      return (z_flag) || (n_flag && !v_flag) || (!n_flag && v_flag);
+
+    /* Higher than (unsigned). */
+    case 13:
+      return (!c_flag) && (!z_flag);
+
+    /* Lower than or same (unsigned). */
+    case 14:
+      return (c_flag) || (z_flag);
+
+    /* Positive non-zero. */
+    case 15:
+      return (!n_flag) && (!z_flag);
+
+    }
+
+  /* Next line is used to supress warning about the return value. It is
+     unreachable during normal execution which does not cause the assertion
+     error, see code above to understand why. */
+  return -1;
+
+}    /* arc_linux_is_condition_true() */
+
+
+/*! Checks result of brcc/bbit instruction condition by instruction arguments
+    and subopcode.
+
+  @param insn BRcc/BBIT{0,1} instruction to analyze. This function doesn't
+              check if insn is indeed a BRcc or BBIT.
+
+  @return Whether condition of this instruction is true or not. */
+static int
+arc_linux_is_brcc_taken (const struct arc_instruction *insn)
+{
+  ULONGEST uarg1, uarg2;
+  LONGEST arg1, arg2;
+  unsigned int subopcode;
+
+  gdb_assert (insn != NULL);
+
+  uarg1 = arc_insn_get_operand_value (insn, 0);
+  uarg2 = arc_insn_get_operand_value (insn, 1);
+  /* These variables will be used for signed comparison. */
+  arg1 = arc_insn_get_operand_value_signed (insn, 0);
+  arg2 = arc_insn_get_operand_value_signed (insn, 1);
+
+  /* 32-bit BRcc/BBIT instructions contains they condition in the
+     subopcode. 16-bit BRcc supports only EQ and NE, but subopcode values for
+     short instruction are identical to those of long instruction. However the
+     field in arc_instruction is different. */
+  subopcode = ((insn->length == 4) ? insn->subopcode3 :
+			    insn->subopcode1);
+
+  gdb_assert (subopcode <= 15);
 
   if (arc_debug)
     {
       fprintf_unfiltered (gdb_stdlog,
-			  "--- arc_linux_next_pc (%s) = %s, isBranch = %s, "
-			  "tcnt = %d [0x%x],\n"
-			  "    flow = %s (%d), reg for indirect jump = %d, "
-			  "nullifyMode = %s\n",
-			  print_core_address (gdbarch, pc),
-			  print_core_address (gdbarch, *fall_thru),
-			  instr.isBranch ? "true" : "false", instr.tcnt,
-			  instr.targets[0],
-			  (instr.flow == direct_jump
-			   || instr.flow == direct_call)
-			  ? "direct" : "indirect", instr.flow,
-			  instr.register_for_indirect_jump,
-			  ((instr.nullifyMode == (char) BR_exec_always)
-			   ? "delay slot" : "no delay"));
+			  " --- arc_linux_is_brcc_taken subopcode = %d,"
+			  "arg1 = %s, arg2 = %s, uarg1 = %s, uarg2 = %s\n",
+			  subopcode, plongest (arg1), plongest (arg2),
+			  pulongest (uarg1), pulongest (uarg2));
     }
 
-  /* OK, it's a branch */
-  if (instr.isBranch)
+  switch (subopcode)
     {
-      two_targets = TRUE;
+    /* BREQ */
+    case 0x0:
+    case 0x8:
+      return uarg1 == uarg2;
 
-      if (instr.flow == direct_jump || instr.flow == direct_call)
+    /* BRNE */
+    case 0x1:
+    case 0x9:
+      return uarg1 != uarg2;
+
+    /* BRLT */
+    case 0x2:
+    case 0xA:
+      return arg1 < arg2;
+
+    /* BRGE */
+    case 0x3:
+    case 0xB:
+      return arg1 >= arg2;
+
+    /* BRLO */
+    case 0x4:
+    case 0xC:
+      return uarg1 < uarg2;
+
+    /* BRHS */
+    case 0x5:
+    case 0xD:
+      return uarg1 >= uarg2;
+
+    /* BBIT0 */
+    case 0x6:
+    case 0xE:
+      return !(uarg1 & (1 << uarg2));
+
+    /* BBIT1 */
+    case 0x7:
+    case 0xF:
+      return (uarg1 & (1 << uarg2));
+    }
+
+  /* This line is used to supress warning about the return value. It is
+     unreachable during normal execution, because assert will catch the case of
+     subopcode > 15. */
+  return -1;
+}    /* arc_linux_is_brcc_taken() */
+
+
+/*! Returns the address of the next instruction or the branch
+    target if it is a branch and it is taken.
+
+    @param[in]	pc */
+static CORE_ADDR
+arc_linux_next_pc (CORE_ADDR pc)
+{
+  struct regcache *regcache = get_current_regcache ();
+  struct gdbarch *gdbarch = get_regcache_arch (regcache);
+  struct arc_instruction insn;
+  int is_loop_enabled;
+  ULONGEST status32;
+  CORE_ADDR next_pc;
+
+  /* So what is the instruction at the given pc? */
+  arc_insn_decode (gdbarch, pc, &insn);
+
+  /* By default, the next instruction is the one immediately after the one at
+     pc. */
+  next_pc = arc_insn_get_linear_next_pc (&insn);
+
+  /* ZD-loops may be disabled in STATUS32.  */
+  regcache_cooked_read_unsigned (regcache, gdbarch_ps_regnum (gdbarch),
+				 &status32);
+  is_loop_enabled = !(status32 & ARC_REG_STATUS32_L);
+
+  if (arc_debug)
+    arc_insn_dump (&insn);
+
+  if (insn.is_control_flow)
+    {
+      int is_branch_taken = FALSE;
+      /* Checking if instruction is brcc or bbit because there is need to
+         calculate branch condition manually then. */
+      int is_brcc_or_bbit = ((insn.opcode == 0x01) && (insn.subopcode1 == 0x1))
+			 || (insn.opcode == 0x1D);
+      int is_branch_lpcc = (insn.opcode == 0x04 && insn.subopcode1 == 0x28);
+
+      if (is_brcc_or_bbit)
 	{
-	  /* It's a direct jump or call, so the destination address is encoded
-         in the instruction, so we got it by disassembling the instruction */
-	  *target = (CORE_ADDR) instr.targets[0];
+	  is_branch_taken = arc_linux_is_brcc_taken (&insn);
+	}
+      /* If branch instruction is not brcc or bbit we have not calculated
+	 branch condition yet and should use status32 register for calculating
+	 it. */
+      else
+	{
+	  /* Non-conditional jumps and branches will have condition_code == 0,
+	     which means "execute always.  */
+	  is_branch_taken =
+	    arc_linux_is_condition_true (status32, insn.condition_code);
+	}
+
+      if (arc_debug)
+	{
+	  fprintf_unfiltered (gdb_stdlog,
+			      "--- arc_linux_next_pc is_brcc_or_bbit = %d, "
+			      "is_branch_taken = %d, is_branch_lpcc = %d\n",
+			      is_brcc_or_bbit, is_branch_taken,
+			      is_branch_lpcc);
+	}
+
+      /* If current instruction is LP(cc) and branch is not taken, next
+         instruction must be calculated the same way as it would if instruction
+         was an another branch or jump and branch was taken. */
+      if ((is_branch_taken && !is_branch_lpcc)
+	  || (!is_branch_taken && is_branch_lpcc))
+	{
+	  next_pc = arc_insn_get_branch_target (&insn);
 	}
       else
 	{
-	  /* It's an indirect jump to the address held in the register
-	     named in the instruction, so we must read that register */
-	  ULONGEST val;
-	  regcache_cooked_read_unsigned (regcache,
-					 instr.register_for_indirect_jump,
-					 &val);
-	  *target = val;
-	}
-
-      /* for instructions with delay slots, the fall thru is not the
-         instruction immediately after the branch instruction, but the one
-         after that. */
-      if (instr.nullifyMode == (char) BR_exec_always)
-	{
-	  struct arcDisState instr_d = arcAnalyzeInstr (*fall_thru, &di);
-
-	  *fall_thru += instr_d.instructionLen;
+	  /* For instructions with delay slots, the fall thru is not the
+	     instruction immediately after the branch instruction, but the one
+	     after that. */
+	  if (insn.has_delay_slot)
+	    {
+	      struct arc_instruction insn_d;
+	      arc_insn_decode (gdbarch, next_pc, &insn_d);
+	      next_pc = arc_insn_get_linear_next_pc (&insn_d);
+	    }
 	}
     }
-
-
-  /* Zero-overhead loops */
-  regcache_cooked_read_unsigned (regcache, ARC_LP_START_REGNUM, &lp_start);
-  regcache_cooked_read_unsigned (regcache, ARC_LP_END_REGNUM, &lp_end);
-  regcache_cooked_read_unsigned (regcache, ARC_LP_COUNT_REGNUM, &lp_count);
-  regcache_cooked_read_unsigned (regcache, gdbarch_pc_regnum (gdbarch), &status32);
-
-  if (!(status32 & 0x00000100) && *fall_thru == lp_end && lp_count > 1)
+  /* Is current instruction the last in a loop body? */
+  else if (is_loop_enabled)
     {
-      /* the instruction is in effect a jump back to the start of the loop */
-      two_targets = TRUE;
-      *target = lp_start;
+      ULONGEST lp_end, lp_start, lp_count;
+      regcache_cooked_read_unsigned (regcache, ARC_LP_START_REGNUM,
+				     &lp_start);
+      regcache_cooked_read_unsigned (regcache, ARC_LP_END_REGNUM, &lp_end);
+      regcache_cooked_read_unsigned (regcache, ARC_LP_COUNT_REGNUM,
+				     &lp_count);
+
+      if (arc_debug)
+	{
+	  fprintf_unfiltered (gdb_stdlog,
+			      "lp_start = %s, lp_end = %s, lp_count = %s, "
+			      "next_pc = %s\n",
+			      pulongest (lp_start), pulongest (lp_end),
+			      pulongest (lp_count), pulongest (next_pc));
+	}
+      if (next_pc == lp_end && lp_count > 1)
+	{
+	  /* The instruction is in effect a jump back to the start of the
+	     loop. */
+	  next_pc = lp_start;
+	}
     }
 
-  return two_targets;
-
+  return next_pc;
 }	/* arc_linux_next_pc () */
 
 
@@ -468,7 +649,7 @@ arc_linux_sigcontext_addr (struct frame_info *this_frame)
 /* -------------------------------------------------------------------------- */
 /*		   ARC specific GDB architectural functions		      */
 /*									      */
-/* Functions are listed in the order they are used in arc_linux_init_abi.     */
+/* Functions are listed in the order they are used in arc_linux_init_osabi.   */
 /* -------------------------------------------------------------------------- */
 
 /*! Determine whether a register can be read.
@@ -559,25 +740,11 @@ arc_linux_breakpoint_from_pc (struct gdbarch *gdbarch, CORE_ADDR * pcptr,
 
 /*! Single step in software.
 
-    Insert breakpoints at all the locations where the program could end up
+    Inserts a breakpoint to the location where the program will end up
     after the current instruction.
 
-    One breakpoint is always the next instruction to be executed (may not be
-    the following instruction if there an unconditional branch). For a
-    conditional branch instruction, there will be a second instruction.
-
-    We don't need to worry about removing the breakpoints - GDB will tidy them
-    up for us automatically. However if first breakpoint has been inserted but
-    the second one fails to insert (for example because of invalid address)
-    then we need to cleanup first breakpoint by ourselves. Otherwise it will be
-    left there and consequent call to insert_single_step_breakpoint will result
-    in a failed assertion in GDB due to inconsistent state. This might happen
-    in there is a code gen error (like bl.d 0), and GDB can do nothing about
-    it, however if first breakpoint is not cleaned up GDB might fail due to an
-    assertion which is not user friendly.
-
     @param[in] frame  The current frame (THIS frame).
-    @return           Non-zero (TRUE) if the breakpoints are inserted
+    @return           Non-zero (TRUE) if the breakpoints are inserted.
  */
 static int
 arc_linux_software_single_step (struct frame_info *frame)
@@ -585,34 +752,11 @@ arc_linux_software_single_step (struct frame_info *frame)
   struct gdbarch *gdbarch = get_frame_arch (frame);
   struct address_space *aspace = get_frame_address_space (frame);
   CORE_ADDR pc = get_frame_pc (frame);
-  CORE_ADDR fall_thru, branch_target;
-  int two_breakpoints = arc_linux_next_pc (pc, &fall_thru, &branch_target);
+  CORE_ADDR next_pc = arc_linux_next_pc (pc);
+  insert_single_step_breakpoint (gdbarch, aspace, next_pc);
 
-  /* No need to check result. If it fails breakpoint hasn't been created and
-   * GDB's internal state will be consistent. */
-  insert_single_step_breakpoint (gdbarch, aspace, fall_thru);
-
-  if (two_breakpoints)
-    {
-      if ((pc != branch_target) && (fall_thru != branch_target))
-	{
-	  /* If second insert fails, first breakpoint has to be removed
-	   * manually. */
-	  TRY
-	    {
-	      insert_single_step_breakpoint (gdbarch, aspace, branch_target);
-	    }
-	  CATCH(ex, RETURN_MASK_ERROR)
-	    {
-	      /* Pass exception further after cleanup. */
-	      delete_single_step_breakpoints (inferior_thread ());
-	      throw_exception(ex);
-	    }
-	  END_CATCH
-	}
-    }
-
-  return 1;			/* returns always true for now */
+  /* Returns always true for now. */
+  return 1;
 
 }	/* arc_linux_software_single_step () */
 
@@ -647,14 +791,16 @@ arc_linux_skip_solib_resolver (struct gdbarch *gdbarch, CORE_ADDR pc)
       if (resolver.minsym)
 	{
 	  CORE_ADDR res_addr = BMSYMBOL_VALUE_ADDRESS (resolver);
-	  fprintf_unfiltered (gdb_stdlog, "--- %s : pc = %s, resolver at %s\n",
-			      __FUNCTION__, print_core_address (gdbarch, pc),
-			      print_core_address (gdbarch, res_addr));
+	  debug_printf ("arc-linux: skip_solib_resolver(): "
+			"pc = %s, resolver at %s\n",
+			print_core_address (gdbarch, pc),
+			print_core_address (gdbarch, res_addr));
 	}
       else
 	{
-	  fprintf_unfiltered (gdb_stdlog, "--- %s : pc = %s, no resolver found",
-			      __FUNCTION__, print_core_address (gdbarch, pc));
+	  debug_printf ("arc-linux: skip_solib_resolver(): "
+			"pc = %s, no resolver found\n",
+			print_core_address (gdbarch, pc));
 	}
     }
 
@@ -679,7 +825,7 @@ arc_linux_supply_gregset (const struct regset *regset,
 			  struct regcache *regcache,
 			  int regnum, const void *gregs, size_t size)
 {
-  const bfd_byte *buf = gregs;
+  const bfd_byte *buf = (const bfd_byte *) gregs;
   unsigned int reg;
 
   for (reg = 0; reg < ARRAY_SIZE (arc_linux_core_reg_offsets); reg++)
@@ -700,7 +846,7 @@ arc_linux_collect_gregset ( const struct regset *regset,
                             void *gregs,
                             size_t size)
 {
-  gdb_byte *buf = gregs;
+  gdb_byte *buf = (gdb_byte *) gregs;
   int reg;
 
   for (reg = 0; reg < ARRAY_SIZE (arc_linux_core_reg_offsets); reg++)
@@ -731,9 +877,9 @@ arc_linux_collect_gregset ( const struct regset *regset,
 
 /* Linux regset definitions. */
 static struct regset arc_linux_gregset = {
-  .regmap         = NULL,
-  .supply_regset  = arc_linux_supply_gregset,
-  .collect_regset = NULL,
+  NULL,
+  arc_linux_supply_gregset,
+  NULL,
 };
 
 /* Implement the `iterate_over_regset_sections` gdbarch method.  */
@@ -751,36 +897,24 @@ arc_linux_iterate_over_regset_sections (struct gdbarch *gdbarch,
 /*                               externally visible functions                 */
 /* -------------------------------------------------------------------------- */
 
+/* Initialization specific to Linux/uClibc environment.  */
 
-/*! Function to determine the OSABI variant.
-
-    Every target variant must define this appropriately.
-
-    @return  The OSABI variant. */
-enum gdb_osabi
-arc_get_osabi (void)
-{
-  return GDB_OSABI_LINUX;
-
-}	/* arc_get_osabi () */
-
-
-/*! Function to initialize for this target variant.
-
-    Every target variant must define this appropriately.
-
-    @param[in,out] gdbarch  The gdbarch we are initializing. */
-void
-arc_gdbarch_osabi_init (struct gdbarch *gdbarch)
+static void
+arc_linux_init_osabi (struct gdbarch_info info, struct gdbarch *gdbarch)
 {
   struct gdbarch_tdep *tdep = gdbarch_tdep (gdbarch);
 
+  if (arc_debug)
+    debug_printf ("arc-linux: GNU/Linux OS/ABI initialization.\n");
+
   /* Fill in target-dependent info in ARC-private structure. */
   tdep->is_sigtramp = arc_linux_is_sigtramp;
   tdep->sigcontext_addr = arc_linux_sigcontext_addr;
   tdep->sc_reg_offset = arc_linux_sc_reg_offset;
   tdep->sc_num_regs = ARRAY_SIZE (arc_linux_core_reg_offsets);
 
+  linux_init_abi (info, gdbarch);
+
   /* Set up target dependent GDB architecture entries. */
   set_gdbarch_cannot_fetch_register (gdbarch, arc_linux_cannot_fetch_register);
   set_gdbarch_cannot_store_register (gdbarch, arc_linux_cannot_store_register);
@@ -800,5 +934,14 @@ arc_gdbarch_osabi_init (struct gdbarch *gdbarch)
 
 }	/* arc_gdbarch_osabi_init () */
 
-/* vim: set sts=2 shiftwidth=2 ts=8: */
+/* Shut up -Wmissing-prototypes.  */
+void _initialize_arc_linux_tdep (void);
 
+void
+_initialize_arc_linux_tdep (void)
+{
+  gdbarch_register_osabi (bfd_arch_arc, 0, GDB_OSABI_LINUX,
+			  arc_linux_init_osabi);
+}
+
+/* vim: set sts=2 shiftwidth=2 ts=8: */
